
#include <stdlib.h>
#include <stdio.h>
#include <ostream.h>

#include "symbol.hh"
#include "types.hh"
#include "typedb.hh"
#include "tree.hh"

#include "oql_context.hh"

#include "desc.hh"

extern "C" {
int getopt();
char *optarg;
}

#include "db.hh"

#include "runtime.hh"

#include "boat.hh"
Boat the_boat;

void browse_type(oqlContext &oql, Type *type)
{
	ContainerType	*con;
	ObjectType	*obj;
	CollectionType	*col;
	PrimitiveType	*prim;
	MethodType	*method;

	cout << "sizeof(" << *type << ") == " << type->size() << endl;

	/*
	   a switch on types would be nice, but
	   then you have to enumerate every known subtype,
	   which sucks
	 */  

	if (obj = type->isObject()) {
		ObjectType *super = obj->superclass();
		cout << "Object: ";
		if (super)
			cout << " isa "  << super->name();
		cout << endl;
		/* XXX fallthrough to container code */
	}

	if (con = type->isContainer()) {
		ContainerType_i *it = con->iterator();
		char	*membername;
		Type	*membertype;
		int	memberoffset;
		if (!obj)
			cout << "Container:" << endl; 
		while (it->next(membername, membertype, memberoffset)) {
			cout << "	[" << membername
				<< "] " << *membertype
				<< " @ " << memberoffset << endl;
//			browse_type(oql, membertype);
		}
		delete	it;
		return;
	}

	if (col = type->isCollection()) {
		cout << "Collection of " << col->of()->name() << endl;
		browse_type(oql, col->of());
		return;
	}

	if (prim = type->isPrimitive()) {
		cout << "Primitive " << type->name() << endl;
		return;
	}

	if (method = type->isMethod()) {
		MethodType_i *it = method->iterator();
		char	*argname;
		Type	*argtype;

		cout << "Method:  returns " << *(method->returns()) << endl; 
		while (it->next(argname, argtype)) {
			cout << "	[\"" << argname
				<< "\"] " << *argtype
				<< endl;
//			browse_type(oql, argtype);
		}
		delete	it;
		return;
	}
}

void browse_name(oqlContext &oql, char *typename)
{
	Type	*type;

	type = oql.db.types.lookup(typename);
	if (!type) {
		cerr.form("browse: %s: type not found\n", typename);
		return;
	}
	browse_type(oql, type);
}

main(int argc, char **argv)
{
	oqlContext	context;
	int	travel = 0;
	char	*to_browse;
	int	optind = 1;

	if (argc < 2) {
		cerr.form("usage: %s [-t] typename\n", argv[0]);
		exit(1);
	}

	if (argc > 2  &&  strcmp(argv[1], "-t") == 0) {
		travel++;
		optind++;
	}

	to_browse = argv[optind];

	init_atomic(context.db);

	if (travel)
		init_travel_db(context.db);
	else
		init_hoofer_db(context.db);

	context.db.types.commit();

	browse_name(context, to_browse);

	exit(0);
}

