#include <ostream.h>
#include "symbol.hh"
#include "typedb.hh"
#include "oql_context.hh"


/*
   Boat {
     id:	integer;
     color:	string;
     sailors:	Set<Sailor>;
     Boats:	Extent<Boat>;
     afloat:	boolean;
     captain:	Ref<Sailor>;
     knots:	Method(real, real);
     gadgets:	List<string>;
     draft:	List<integer>;
   }

   Sailor {
     id:	integer;
     name:	string;
     hair_color:string;
     boats:	Set<Boat>;
     Sailors:	Extent<Sailor>;
   }

   Top level method (Boat class method) ...
   Available(color: string) : Set<Boats>;
 */

void init_hoofer_db(oqlDatabase &db)
{
	TypeDB *Types = &db.types;
	ExtentDB *Extents = &db.extents;
	Type *_char;
	Type *_integer;
	Type *_string;
	Type *_real;
	Type *_boolean;
	Type *tmp;

	_char = Types->force_lookup("char");
	_integer = Types->force_lookup("integer");
	_real = Types->force_lookup("real");
	_string = Types->force_lookup("string");
	_boolean = Types->force_lookup("boolean");

	/* Build a complex type */
	ObjectType *boat = new ObjectType("Boat", (ObjectType *)0);
	Types->add(boat);
	boat->add("id", _integer);
	boat->add("color", _string);
	boat->add("afloat", _boolean);

	/* Build another complex type */
	ObjectType *sailor = new ObjectType("Sailor");
	Types->add(sailor);

	// tmp = new CollectionType("Ref<Sailor>", tt_ref, sailor);
	tmp = new SetType(sailor);
	boat->add("sailors", tmp);
	Extents->add("Sailors", tmp);

	sailor->add("id", _integer);
	sailor->add("name", _string);
	sailor->add("hair_color", _string);

#ifdef need_refs
	tmp = new RefType("Ref<Sailor>", sailor);
#else
	tmp = sailor;
#endif
	boat->add("captain", tmp);

	// tmp = new CollectionType("Ref<Boat>", tt_ref, boat);
	tmp = new SetType(boat);
	sailor->add("boats", tmp);
	Extents->add("Boats", tmp);

	MethodType *m;
	m = new MethodType(_real);
	m->addArg("weight", _real);
	boat->add("knots", m);

	boat->add("gadgets", new ListType(_string));
	boat->add("draft", new ListType(_integer));

	ObjectType *sailboat = new ObjectType("Sailboat", boat);
	sailboat->add("masts", _integer);
	sailboat->add("canvas", _real);	/* sq ft of canvas :-) */
	Types->add(sailboat);
	tmp = new SetType(sailboat);
	Extents->add("Sailboats", tmp);

	m = new MethodType(new SetType(boat));
	m->addArg("color", _string);
	Extents->add("Available", m);

	/* Make some typed named objects */
#ifdef need_refs
	Extents->add("fin", new RefType(boat));
	Extents->add("m20", new RefType(boat));
	Extents->add("david", new RefType(sailor));
	Extents->add("bolo", new RefType(sailor));
	Extents->add("clipper", new RefType(sailboat));
#else
	Extents->add("fin", boat);
	Extents->add("m20", boat);
	Extents->add("david", sailor);
	Extents->add("bolo", sailor);
	Extents->add("clipper", sailboat);
#endif
	Extents->add("xclipper", sailboat);
	Extents->add("xm20", boat);
}
	
