
#include <ostream.h>
#include <strstream.h>
#include "symbol.hh"
#include "types.hh"

#include <assert.h>

PrimitiveType TypeVoid("void", PrimitiveType::pt_void);
ErrorType TypeError("*ERROR*");
ObjectType TypeNil("nil");


ostream &operator<<(ostream &s, Type &type)
{
	return type.print(s);
}

#if 0
#include "primitive_names.h"
ostream &PrimitiveType::print(ostream &s)
{
	return s << '<' << PrimitiveNames[_primitive] << '>';
}
#endif

ostream &MethodType::print(ostream &s)
{
	int	i;
	Symbol	*sym;
	s << "<Method " << name() << '(';
	for (i = 0; sym = args.iterate(i); ) {
		s << sym->name() << ':' << *(Type *)(sym->data());
		if (i != -1)
			s << ", ";
	}
	return s << ") : " << _returns->name() << ">";
}


ostream &ContainerType::_print(ostream &s, char *typename)
{
	int	token = 0;
	Symbol	*sym;
	Type	*type;

	if (_name) {
		/* kludge for objects */
		if (typename)
			return s << '<' << typename << '<' << _name << ">>";
		else
			return s << '<' << _name << '>';
	}
	
	s << '<' << typename << " <";

	while (sym = members.iterate(token)) {
		s << sym->name() << ':';
		type = (Type *) (sym->data());
		s << type->name();
		if (token >= 0)
			s << ' ';
	}
	s << ">>";
	return s;
}

ostream &CollectionType::_print(ostream &s, char *typename)
{
	return s << '<' << typename <<  *_of << ">";
}

Type	*RefType::rebuild(Type *of)
{
	return new RefType(of);
}

Type	*RefType::lcs(Type *_it)
{
	RefType *it = _it->isRef();
	Type	*of_lcs;

	if (!it)
		return 0;

	of_lcs = _of->lcs(it->_of);
	if (!of_lcs)
		return (Type *)0;

	return rebuild(of_lcs);
}

Type	*RefType::canCompareWith(Type *_it)
{
	return lcs(_it);
}

ostream &RefType::print(ostream &s)
{
	return s << "<Ref" << *_of << '>';
}

Type	*PrimitiveType::lcs(Type *_it)
{
	PrimitiveType *it = _it->isPrimitive();

	if (!it)
		return (Type *)0;

	if (_primitive == it->_primitive)
		return this;
	if ((_primitive == pt_real || _primitive == pt_integer) &&
	    (it->_primitive == pt_real  ||  it->_primitive == pt_integer)) {
		/* return an existing type, don't look it up in db! */
		if (_primitive == pt_real)
			return this;
		else
			return it;
	}
	return (Type *)0;
}

Type	*PrimitiveType::canCompareWith(Type *_it)
{
	return lcs(_it);
}

Type	*PrimitiveType::canDoMathWith(Type *_it)
{
	return lcs(_it);
}

Type	*PrimitiveType::canDoMath()
{
	return (_primitive == pt_real ||  _primitive == pt_integer) ?
		this : 0;
}

Type	*ContainerType::canCompareWith(Type *_it)
{
	return lcs(_it);
}

int	ObjectType::isa(ObjectType *it)
{
	if (it == this)
		return	1;
	if (!_superclass)
		return 0;
	return _superclass->isa(it);
}

void ObjectType::classlist(ObjectType **list, int &where, int number_left)
{
	assert(number_left > 0);
	list[where++] = this;
	if (_superclass)
		_superclass->classlist(list, where, number_left-1);
}

Type	*ObjectType::lcs(Type *_it)
{
	ObjectType *super;
	ObjectType *it;
	ObjectType **my_classes;
	ObjectType **its_classes;
	int	my_n, its_n;
	int	i, j;
	ObjectType *result;

	it = _it->isObject();
	if (!it)
		return (Type *)0;
	
	my_classes = new ObjectType *[100];
	my_n = 0;
	classlist(my_classes, my_n, 100);
	its_classes = new ObjectType *[100];
	its_n = 0;
	it->classlist(its_classes, its_n, 100);

	for (i = 0; i < my_n; i++)
		for (j = 0; j < its_n; j++)
			if (my_classes[i] == its_classes[j]) {
				result = my_classes[i];
				delete my_classes;
				delete its_classes;
				return result;
			}
	delete my_classes;
	delete its_classes;
	return (ObjectType *)0;
}


/* Add the tuple entries in the 'from' tuple to our tuple */
void	TupleType::concat(TupleType &from)
{
	int	token = 0;
	Symbol	*sym;

	/* XXX assumes no collisions */
	while (sym = from.members.iterate(token))
		add(sym->name(), (Type *)(sym->data()));
}

/*
   The LCS of two tuples, is the intersection of
   all attributes of the same name which have types which
   have LCSs
  */
Type	*TupleType::lcs(Type *_it)
{
	TupleType	*it = _it->isTuple();
	Type	*one,*two;
	int	i, j;
	Symbol	*is, *js;
	TupleType	*result = (TupleType *)0;
	Type	*lcs;

	if (!it)
		return (Type *)0;

	i = 0;
	while (is = members.iterate(i)) {
		j = 0;
		while (js = it->members.iterate(j)) {
			if (strcmp(is->name(), js->name()) != 0)
				continue;
			one = (Type *)(is->data());
			two = (Type *)(js->data());
#if 0
			lcs = ((Type *)(is->data()))->lcs(((Type *)(js->data())));
#else
			lcs = one->lcs(two);
#endif
			if (lcs) {
				cout << "Combine " << *one <<
					", " << *two <<'\n';
				cout << "\tInto " << *lcs << '\n'; 
				if (!result)
					result = new TupleType();
				result->add(is->name(), lcs);
			}
		    }
	}
	return result;
}

Type	*CollectionType::_lcs(CollectionType *it)
{
	Type	*of_lcs;

	/* CollectionType match already done by downwards lcs()s */

	of_lcs = _of->lcs(it->_of);
	if (!of_lcs)
		return (Type *)0;

	return rebuild(of_lcs);
}

Type	*BagType::lcs(Type *_it)
{
	BagType *it = _it->isBag();

	if (!it)
		return 0;

	return CollectionType::_lcs(it);
}

Type	*SetType::lcs(Type *_it)
{
	SetType *it = _it->isSet();

	/* allow set->bag downgrades, needed because of l->rebuild(r)  */
	if (!it && _it->isBag())
		return _it->lcs(this);

	return BagType::lcs(it);
}

Type	*ListType::lcs(Type *_it)
{
	ListType *it = _it->isList();

	if (!it)
		return 0;
	return CollectionType::_lcs(it);
}

Type	*ArrayType::lcs(Type *_it)
{
	ArrayType *it = _it->isArray();

	if (!it)
		return 0;
	return CollectionType::_lcs(it);
}


#if 1
char	*TupleType::name()
{
#if 0
//	return Type::name();
	char buf[1024];
	char buf2[128];
	Symbol *sym;
	int	token = 0;

	strcpy(buf, "<Tuple<");

	while (sym = members.iterate(token)) {
		sprintf(buf2, "%s:%s%s",
			sym->name(),
			((Type *)(sym->data()))->name(),
			token >= 0 ? " " : "");
		strcat(buf, buf2);
	}
	return strdup(strcat(buf, ">>"));	/* XXX leak */
#else
	ostrstream buf;
	buf << *this;
	return buf.str();	/* XXX leak */
#endif
}
#endif

char	*Type::name()
{
	if (_name)
		return _name;
#if 0
	return strdup(form("#%#x", (int)this));
#else
//	cout << "Type::name() using tricks\n";
	ostrstream buf;
	buf << *this;
	return buf.str();	/* XXX leak */
#endif
}
