#include <stdlib.h>
#include <ostream.h>
#include <string.h>

#include "symbol.hh"

bool	Symbol::matches(char *name)
{
	return strcmp(name, _name) == 0;
}

SymbolTable::~SymbolTable()
{
	int	i;

	for (i = 0; i < _size; i++) {
		delete syms[i];
		syms[i] = (Symbol *)0;
	}
	_size = 0;
}

int SymbolTable::_lookup(char *name)
{
	int	i;

	for (i = _size-1; i >= 0 ; i--) {
		if (syms[i]->matches(name))
			return i;
	}
	return -1;
}

ostream &SymbolTable::print(ostream &s)
{
	int	i;

	s << "SymbolTable(" << _name
		<< ") (" << _size << " entries) {\n";
	for (i = 0; i < _size; i++)
		s << '\t' << *syms[i] << '\n';
	return s << "}\n";
}

Symbol *SymbolTable::lookup(char *name)
{
	int	offset;

	offset = _lookup(name);
	return (offset == -1) ? ((Symbol *)0) : syms[offset];
}

Symbol *SymbolTable::_enter(char *name, void *data)
{
	Symbol	*s;

	if (_size >= SIZE) {
		cerr << "symbol table " << name << " out of space\n";
		return (Symbol *)0;
	}
	s = new Symbol(name, data, this);
	syms[_size++] = s;
	return s;
}

Symbol	*SymbolTable::overwrite(char *name, void *data)
{
	Symbol	*s;

	s = lookup(name);
	if (s) {
		s->setData(data);
		return s;
	}
	return _enter(name, data);
}


Symbol *SymbolTable::enter(char *name, void *data, int do_overwrite)
{
	return do_overwrite ? overwrite(name, data) : _enter(name, data);
}

void SymbolTable::_remove(int offset, int zap)
{
	int	i;

	if (zap)
		delete syms[offset];
	for (i = offset; i < _size-1; i++)
		syms[i] = syms[i+1];
	_size--;
	syms[_size] = (Symbol *)0;
}

int SymbolTable::remove(Symbol *sym)
{
	int	offset;

	offset = _lookup(sym->name());
	if (offset == -1  ||  syms[offset] != sym)
		return -1;

	_remove(offset, 0);
	return 0;
}
	
int SymbolTable::remove(char *name)
{
	int offset = _lookup(name);

	if (offset == -1)
		return -1;
	_remove(offset, 1);
	return 0;
}

Symbol *SymbolTable::iterate(int &token)
{
	Symbol *result;
	if (token < 0 || token >= _size)
		return (Symbol *)0;
	result = syms[token++];
	if (token >= _size)		/* give clue of last entry */
		token = -1;
	return result;
}

ostream &operator<<(ostream &s, Symbol &sym)
{
	return s << "<" << sym._name << ", " << int(sym._data) << ">";
}

ostream &operator<<(ostream &s, SymbolTable &table)
{
	return table.print(s);
}
