#ifndef _SYMBOL_H_
#define _SYMBOL_H_

#include "auto_string.hh"

class SymbolTable;
class Symbol {
	friend ostream &operator<<(ostream &s, Symbol &sym);
private:
	auto_string	_name;
	void	*_data;
	int	_flags;
	SymbolTable	*_table;
	
public:
	Symbol(char *name,
	       void *data = 0,
	       SymbolTable *table = (SymbolTable *)0) : _name(name) {
		_data = data;
		_table = table;
		_flags = 0;
	}
	~Symbol() {
		_data = 0;
	}
	
	char	*name()  { return _name.str(); }
	void	*data()	{ return _data; }
	SymbolTable	*table() { return _table; }
	void	setData(void *data) { _data = data; }
	void	setTable(SymbolTable *table) { _table = table; }

	void	setFlags(int flags) { _flags = flags; }
	int	flags() { return _flags; }
	
	bool	matches(char *name);
};

ostream &operator<<(ostream &s, Symbol &sym);

class SymbolTable {
	friend	ostream &operator<<(ostream &s, SymbolTable &t);
private:
	const	int	SIZE = 100;
	auto_string	_name;
	Symbol	*syms[SIZE];
	int	_size;

	int	_lookup(char *name);
	void	_remove(int offset, int zap = 1);
	ostream	&print(ostream &s);

public: /* protected: */
	Symbol	*iterate(int &token);
	Symbol	*_enter(char *name, void *data = 0);

public:
	SymbolTable(char *name = "<noname>") : _name(name) {
		_size = 0;
		for (int i = 0; i < SIZE; i++)
			syms[i] = 0;
	}
	~SymbolTable();

	Symbol	*overwrite(char *name, void *data = 0);
	Symbol	*enter(char *name, void *data = 0, int overwrite = 0);
	Symbol	*lookup(char *name);
	int	remove(Symbol *sym);
	int	remove(char *name);
	char	*name() { return _name; }
};
	
class SymbolTable_i {
	SymbolTable	&_table;
	int		_token;
public:
	SymbolTable_i(SymbolTable &table) : _table(table) {
		_token = 0;
	}
	Symbol	*next() { return _table.iterate(_token); } 
};

ostream &operator<<(ostream &s, Symbol &s);
ostream &operator<<(ostream &s, SymbolTable &t);

#endif /* _SYMBOL_H_ */
