#include <ostream.h>
#include "symbol.hh"
#include "typedb.hh"

#include "oql_context.hh"

void init_travel_db(oqlDatabase &db)
{
	TypeDB *Types = &db.types;
	ExtentDB *Extents = &db.extents;
	Type *_char;
	Type *_integer;
	Type *_string;
	Type *_real;
	Type *_boolean;
	StructType *t;
	MethodType *m;


	_char = Types->force_lookup("char");
	_integer = Types->force_lookup("integer");
	_real = Types->force_lookup("real");
	_string = Types->force_lookup("string");
	_boolean = Types->force_lookup("boolean");

	Type *date = _string;	/* kludge-o-matic */


	ObjectType *country = new ObjectType("Country");
	country->add("name", _string);
	

	ObjectType *city = new ObjectType("City");
	city->add("name", _string);
	city->add("country", country);


	ObjectType *address = new ObjectType("Address");
	address->add("country", country);
	address->add("zip_code", _string);
	address->add("city", city);
	address->add("street", _string);
	address->add("number", _integer);


	ObjectType *ptg = new ObjectType("Place_to_go");
	ptg->add("name", _string);
	ptg->add("address", address);
	ptg->add("picture", new ArrayType(_char));	/* XXX gross */


	ObjectType *person = new ObjectType("Person");
	person->add("name", _string);
	person->add("first_name", _string);
	person->add("birthdate", date);
#ifdef bogus_odl
	t = new StructType("address");
	t->add("country", country);
	t->add("zip_code", _string);
	t->add("city", city);
	t->add("street", _string);
	t->add("number", _integer);
#endif
	person->add("address", address);
	

	ObjectType *client = new ObjectType("Client", person);
	client->add("bill", _real);
	client->add("place_of_stay", city);
	client->add("departure_date", date);
	client->add("returning_date", date);
	client->add("number_of_persons", _integer);


	ObjectType *employee = new ObjectType("Employee", person);
	employee->add("seniority", _integer);
	employee->add("title", _string);
	employee->add("salary", _real);


	/* No multiple inheritance for Client_Employee (yet) */


	ObjectType *reservation = new ObjectType("Reservation");
	reservation->add("client", client);
	reservation->add("date_of_reservation", date);
	reservation->add("number_of_persons", _integer);


	ObjectType *hotel = new ObjectType("Hotel", ptg);
	hotel->add("name", _string);
	hotel->add("address", address);
	hotel->add("facilities", new SetType(_string));
	hotel->add("stars", _integer);
	hotel->add("city", city);
	t = new StructType;	/* struct ?? */
	t->add("breakfast_price", _real);
	t->add("half_board_price", _real);
	t->add("full_board_price", _real);
	hotel->add("day_price", t);
	t = new StructType("room");	/* struct room */
	t->add("bed_number", _integer);
	t->add("room_number", _integer);
	hotel->add("rooms", new SetType(t));


	ObjectType *hotel_rv = new ObjectType("Hotel_reservation",
					      reservation);
	hotel_rv->add("end_date", date);
	hotel_rv->add("selected_hotel", hotel);
	hotel_rv->add("type_of_stay", _string);
	

	ObjectType *museum = new ObjectType("Museum", ptg);
	museum->add("artists", new SetType(person));

	ObjectType *monument = new ObjectType("Monument", ptg);
	monument->add("architect", person);
	monument->add("building_date", date);

	ObjectType *theater = new ObjectType("Theater", ptg);
	theater->add("building_date", date);
	theater->add("number_of_seats", _integer);

	StructType *stage = new StructType("stage");
	stage->add("what", ptg);
	stage->add("time", _string);

	ObjectType *tour = new ObjectType("Tour");
	tour->add("price", _real);
	tour->add("description", new ListType(stage));
	tour->add("capacity", _integer);
	tour->add("availability", _integer);
	tour->add("theme", _string);
	tour->add("city", city);
	tour->add("title", _string);


	ObjectType *tour_rv = new ObjectType("Tour_reservation", reservation);
	tour_rv->add("selected_tour", tour);
	m = new MethodType(_boolean);
	m->addArg("departure", date);
	m->addArg("back", date);
	tour_rv->add("available", m);

	country->add("cities", new SetType(city));

	city->add("hotels", new SetType(hotel));
	city->add("places_to_go", new SetType(ptg));
	city->add("tours", new SetType(tour));

	client->add("reservations", new ListType(reservation));

	Types->add(country);
	Types->add(city);
	Types->add(address);
	Types->add(ptg);
	Types->add(person);
	Types->add(client);
	Types->add(hotel);
	Types->add(museum);
	Types->add(monument);
	Types->add(theater);
	// Types->add(stage);
	Types->add(tour);
	Types->add(tour_rv);
	Types->add(hotel_rv);
	Types->add(employee);
	Types->add("tour_grp", new SetType(tour));

	Extents->add("The_clients", new SetType(client));
	Extents->add("Hotel", new SetType(hotel));
	Extents->add("Places", new SetType(ptg));
	Extents->add("Monument", new SetType(monument));
	Extents->add("Museum", new SetType(museum));
	Extents->add("Theater", new SetType(theater));
	Extents->add("Country", new SetType(country));
	Extents->add("City", new SetType(city));
	Extents->add("Tour", new SetType(tour));
	Extents->add("Reservations_made", new SetType(reservation));
}
	
