#include <ostream.h>
#include <stdio.h>
#include "ql_if.h"
#include "tree.hh"

void Ql_tree_node::init(NodeType type, int kids)
{
	_type = type;

	_nkids = kids;
	if (kids) {
		_kids = new Ql_tree_node *[kids];
		for (int i = 0; i < kids; i++)
			_kids[i] = 0;
	}
	else
		_kids = (Ql_tree_node **)0;
}


Ql_tree_node::Ql_tree_node(NodeType type, char *id) : _id(id, true)
{
	init(type, 0);
}

Ql_tree_node::Ql_tree_node(NodeType type, int nkids,
			   Ql_tree_node *one,
			   Ql_tree_node *two,
			   Ql_tree_node *three,
			   Ql_tree_node *four)
{
	init(type, nkids);
	switch (nkids) {
	case 4:	_kids[3] = four;
	case 3:	_kids[2] = three;
	case 2:	_kids[1] = two;
	case 1:	_kids[0] = one;
	}
}

Ql_tree_node::~Ql_tree_node()
{
	int	i;

	/* Free kids first ; they can have reference to our info */
	for (i = 0; i < _nkids; i++) {
		delete _kids[i];
		_kids[i] = (Ql_tree_node *)0;
	}
	/* Free the pointers to the kids */
	if (_nkids)
		delete _kids;
}

Ql_tree_node *Ql_tree_node::addLeft(Ql_tree_node *son)
{
	Ql_tree_node **old, **next;
	int	i;

	old = _kids;
	next = new Ql_tree_node *[_nkids+1];

	next[0] = son;
	for (i = 0; i < _nkids; i++)
		next[i+1] = old[i];
	_nkids++;
	_kids = next;
	delete old;

	return this;
}

#if 0
/* Steal the mallocated string from the node */ 
char *Ql_tree_node::steal()
{
#if 0
	char	*id = _id;
	_id = (char *)0;
	return id;
#else
	/* Can't steal the id; some parts of the tree need to be
	   converted MULTIPLE times ! */
	if (!_id || !_id[0])
		cerr.form("Warning: stealing an empty id from tree %#lx\n",
			  (long )this);
	return strdup(_id);
#endif
}
#endif

static char *indent(int level)
{
	static char buf[1024];
	static char spaces[] = "    ";
	char 	*s = buf;
	int	l = strlen(spaces);
	int	i;

	for (i = 0; i < level; i++) {
		strcpy(s, spaces);
		s += l;
	}
	*s = '\0';
	return buf;
}

ostream &Ql_tree_node::print(ostream &s, int level)
{
	int	i;

	if (_nkids == 0) {
		s << form("[%s], [id = %s]\n", node_name(_type),  _id.str());
		return s;
	}

	s << form("[%s] ", node_name(_type)) << form(" %d kids:\n", _nkids);

	for (i = 0; i < _nkids; i++) {
		s << indent(level+1) <<  form("[%d] ", i);
		_kids[i]->print(s, level+1);
	}
	return s;
}

ostream &operator<<(ostream &s, Ql_tree_node *node)
{
	return node->print(s);
}

ostream &operator<<(ostream &s, Ql_tree_node &node)
{
	return node.print(s);
}

#include "node_names.h"

char *node_name(NodeType type)
{
	static char errbuf[128];
	if (type < 0 || type >= num_node_names) {
		sprintf(errbuf, "<unknown %d>", type);
		return errbuf;
	}
	return node_names[type];
}

