#ifndef _TREE_H_
#define _TREE_H_

#include "node_types.h"
#include "symbol.hh"
#include "types.hh"
#include "aqua.hh"
#include "typedb.hh"

class oqlContext;
class translationState;

typedef enum node_type NodeType;

class  Ql_tree_node {
	friend ostream &operator<<(ostream &s, Ql_tree_node *node);
	friend ostream &operator<<(ostream &s, Ql_tree_node &node);
private:
	NodeType	_type;
	Ql_tree_node	**_kids;
	int	_nkids;
	auto_string	_id;
	void	init(NodeType type, int nkids);
	ostream	&print(ostream &s, int level = 0);

	// char	*steal();	// dups the id
	char	*id() { return _id; }

	aqua_t	*convert(translationState &);
	aqua_t	*convert_zary(translationState &);
	aqua_t	*convert_trinary(translationState &);
	aqua_t	*convert_binary(translationState &);
	aqua_t	*convert_unary(translationState &);
	aqua_t	*convert_constant(translationState &);
	aqua_t	*convert_list(translationState &);
	aqua_t	*convert_select(translationState &);
	aqua_t	*convert_object(translationState &);
	aqua_t	*convert_one_scan_var(translationState &);
	aqua_t	*convert_id(translationState &);
	aqua_t	*convert_define(translationState &);
	aqua_t	*convert_existential(translationState &);
	aqua_t	*expand_lambda(Symbol *sym);
	aqua_t	*convert_collection(translationState &);
	aqua_t	*convert_parameters(translationState &);
	aqua_t	*convert_sort(translationState &);
	aqua_t	*convert_group(translationState &);
	aqua_t	*convert_folded(translationState &);	/* XXX right word */
	aqua_t	*convert_method(translationState &);

	void	enter_symbols(translationState &);

public:
	~Ql_tree_node();
	Ql_tree_node(NodeType type, char *id);
	Ql_tree_node(NodeType type, int nkids,
		     Ql_tree_node *one,
		     Ql_tree_node *two = 0, Ql_tree_node *three = 0,
		     Ql_tree_node *four = 0);
	Ql_tree_node	*addLeft(Ql_tree_node *what);
	aqua_t	*aqua_convert(oqlContext &context);
};

extern char *node_name(NodeType nt);
#endif /* _TREE_H_ */
